/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2009 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "fixedFluxBuoyantExnerFvPatchScalarField.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "ExnerTheta.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

fixedFluxBuoyantExnerFvPatchScalarField::
fixedFluxBuoyantExnerFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedGradientFvPatchScalarField(p, iF)
{}


fixedFluxBuoyantExnerFvPatchScalarField::
fixedFluxBuoyantExnerFvPatchScalarField
(
    const fixedFluxBuoyantExnerFvPatchScalarField& ptf,
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedGradientFvPatchScalarField(ptf, p, iF, mapper)
{}


fixedFluxBuoyantExnerFvPatchScalarField::
fixedFluxBuoyantExnerFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const dictionary& dict
)
:
    fixedGradientFvPatchScalarField(p, iF, dict)
{
//    gradient() = 0;
//    fvPatchField<scalar>::operator=(patchInternalField());
}


fixedFluxBuoyantExnerFvPatchScalarField::
fixedFluxBuoyantExnerFvPatchScalarField
(
    const fixedFluxBuoyantExnerFvPatchScalarField& wbppsf
)
:
    fixedGradientFvPatchScalarField(wbppsf)
{}


fixedFluxBuoyantExnerFvPatchScalarField::
fixedFluxBuoyantExnerFvPatchScalarField
(
    const fixedFluxBuoyantExnerFvPatchScalarField& wbppsf,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedGradientFvPatchScalarField(wbppsf, iF)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void fixedFluxBuoyantExnerFvPatchScalarField::updateCoeffs()
{
    if (updated())
    {
        return;
    }

    const dictionary& environmentalProperties
        = db().lookupObject<IOdictionary>("environmentalProperties");

    const dictionary& thermoProperties
        = db().lookupObject<IOdictionary>("thermophysicalProperties");

    dimensionedVector g(environmentalProperties.lookup("g"));

    const constTransport<hConstThermo<perfectGas<specie> > > air
    (
         thermoProperties.lookup("mixture")
    );

    const dimensionedScalar Cp("Cp", dimGasConstant, air.cp(0,0)/air.W());

    const fvsPatchField<scalar>& thetaf =
        patch().lookupPatchField<surfaceScalarField, scalar>("thetaf");

    gradient() = (g.value() & patch().nf())/(Cp.value()*thetaf);
    
    fixedGradientFvPatchScalarField::updateCoeffs();
}


void fixedFluxBuoyantExnerFvPatchScalarField::write(Ostream& os) const
{
    fixedGradientFvPatchScalarField::write(os);
    writeEntry("value", os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField
(
    fvPatchScalarField,
    fixedFluxBuoyantExnerFvPatchScalarField
);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
